<?php
$Pascalia = FALSE;

function getEasterDate( $year )
{
	global $Pascalia;

	if( !$Pascalia )
	{
        $dir = dirname(__FILE__);
		$Pascalia = file("$dir/../date/pascalia.csv" );
	}
	$minYear = 0;
	$maxYear = 0;
	$EasterDate = FALSE;
	foreach( $Pascalia as $line )
	{
		$date = rtrim( $line );
		$date = strtotime( $date );
		if( !$date || $date == -1 )
		{
			break;
		}
		$date = getdate( $date );
		$dateYear = $date["year"];
		if( empty( $minYear ) )
		{
			$minYear = $dateYear;
		}
		$maxYear = $dateYear;
		if( $dateYear == $year )
		{
			$EasterDate = $date;
			break;
		}
	}
	if( $EasterDate )
	{
		$EasterDate["minYear"] = $minYear;
		$EasterDate["maxYear"] = $maxYear;
	}
	return $EasterDate;
}

function getEasterWeek( $date = FALSE )
{
	if( !$date )
	{
		$date = time();
	}
	$zi = getdate( $date );
	$year = $zi["year"];
	$Easter = getEasterDate( $year );
	$week = getWeekFromEaster( $zi, $Easter );
	return $week;
}

// Intoarce nr. zilei dupa Pasti
function getWeekFromEaster( $zi, $Easter )
{
	if( !$Easter )
	{
		return FALSE;
	}
	assert( $Easter["wday"] === 0 );
	assert( $Easter["year"] == $zi["year"] ||
        $Easter["year"] == $zi["year"] - 1 && !$zi["yday"]);
	$week = getSpecialWeekend( $zi, $Easter );
	if( $week && !is_numeric( $week ) ) // Zi speciala (bd*, sc*, nd*)
	{
		return $week;
	}
	$wday = $zi["wday"];
	if( $wday == 0 ) // Duminicile, cf. cu Evanghelistarul
	{
		$week = getSundayFromEaster( $zi, $Easter );
		return $week;
	}
	$yday = $zi["yday"];
	$dayEaster = $Easter["yday"];
	$week = $yday - $dayEaster;
	if( $week < -76 ) // @todo: unde se face intoarcerea Evangheliei ?
	{
		$delta = 7 - $wday; // zile pana la duminica urmatoare
		$duminica = $zi[0] + $delta * 86400;
		$duminica = getdate( $duminica );
		$duminica = getSundayFromEaster( $duminica, $Easter );
		if( !is_numeric( $duminica ) )
		{
			$week += 350 + 14; // Ajustare 2008 ?!?
		}
		else
		{
			$week = $duminica - $delta;
		}
	}
	return $week;
}

// Intoarce nr. zilei de duminica dupa Pasti
function getSundayFromEaster( $zi, $Easter )
{
	$wday = $zi["wday"];
	assert( $wday === 0 );
	$week = getSpecialWeekend( $zi, $Easter );
	if( $week !== FALSE )
	{
		return $week;
	}
	$day = $zi["yday"];
	$dayEaster = $Easter["yday"];
	$week = $day - $dayEaster;
	if( $week < -76 ) // Inainte de Triod
	{
		$diff = -$week - 70; // Distanta in zile pana la Triod
		$week = $diff / 7; // Distanta in saptamani
		$week = getBDWeek( $week, $Easter ); // Saptamana dupa Rusalii
		$week = 7 * ( $week + 7 ); // Ziua dupa Pasti
	}
	return $week;
}

// Intoarce nr. duminicii/sambetei speciale sau FALSE
function getSpecialWeekend( $zi, $Easter )
{
	$duminica = $zi["wday"] === 0;
	$sambata = $zi["wday"] === 6;
    $dir = dirname(__FILE__);
	$file = fopen( "$dir/../date/duminici_speciale.csv", "r" );
	while( ( $day = fgetcsv( $file, 80 ) ) !== FALSE )
	{
		// 0 = luna, 1 = zi,
		// 2 = sambata dinainte, 3 = duminica dinainte,
		// 4 = ziua exacta,
		// 5 = sambata dupa, 6 = duminica dupa
		assert( count( $day ) == 7 );
		$mday = $day[1];
		$month = $day[0];
		$exact = $day[4];
		if( $duminica )
		{
			$before = $day[3];
			$after = $day[6];
		}
		elseif( $sambata )
		{
			$before = $day[2];
			$after = $day[5];
		}
		$week = checkSpecial( $zi, $mday, $month, @$before, $exact, @$after, $Easter );
		if( $week )
		{
			break;
		}
	}
	fclose( $file );
	if( $week )
	{
		if( is_numeric( $week ) ) // Zi nespeciala
		{
			$week = 7 * ( $week + 7 ); // Ziua dupa Pasti
		}
		return $week;
	}
	return FALSE;
}

// Intoarce nr. duminicii dupa Rusalii sau bd*, sc*, nd* sau FALSE
function checkSpecial( $zi, $day, $month, $week_before, $week_exact, $week_after, $Easter )
{
	$holiday = mktime( 0, 0, 0, $month, $day, $zi["year"] );
	$holiday = getdate( $holiday );
	$wday = $holiday["wday"];
	$holiday = $holiday["yday"];
	$zi = $zi["yday"];
	$diff = $zi - $holiday;
	if( $diff == 0 )
	{
		return $wday ? FALSE : $week_exact; // Exact only on Sundays
	}
	if( $diff < 0 && $diff >= -7 )
	{
		return $week_before;
	}
	if( $diff > 0 && $diff <= 7 )
	{
		return $week_after;
	}
	if( $diff > 0 )
	{
		$diff--;
	}
	$diff = floor( $diff / 7 );
	if( $week_exact == "nd" )
	{
		$diff = -$diff - 1;
	}
	if( $diff <= 0 )
	{
		return FALSE;
	}

    $dir = dirname(__FILE__);
	$file = fopen( "$dir/../date/duminici_variabile.csv", "r" );
	$week = FALSE;
	while( ( $day = fgetcsv( $file, 80 ) ) !== FALSE )
	{
		$size = count( $day );
		assert( $size > 1 );
		if( $day[0] != $week_exact )
		{
			continue;
		}
        
		if( $diff < $size )
		{
			if( $week_exact == "bd" ) {
				$week = getBDWeek( $diff, $Easter );
			} else {
				$week = $day[$diff];
			}
			break;
		}
		elseif( $week_exact == "sc" && $diff < 12 )
		{
			$diff -= $size - 1;
			$size = $wday < 4 ? 5 : 6;
			$week = checkSC( $size, $diff, $Easter );
			if( $week )
			{
				break;
			}
		}
	}
	fclose( $file );
	return $week;
}

function checkSC( $size, $diff, $Easter )
{
    $dir = dirname(__FILE__);
	$file = fopen( "$dir/../date/duminici_variabile_sc.csv", "r" );
	$week = FALSE;
	while( ( $day = fgetcsv( $file, 80 ) ) !== FALSE )
	{
		assert( count( $day ) == $day[0] + 1 );
		if( $day[0] == $size && $diff < count( $day ) )
		{
			$week = $day[$diff];
			break;
		}
	}
	fclose( $file );
	if( $week == "?" )
	{
		$week = checkSC_BD( $Easter );
		assert( $week !== FALSE );
	}
	return $week;
}

function checkSC_BD( $Easter )
{
	$size = getBDSize( $Easter );
    $dir = dirname(__FILE__);
	$file = fopen( "$dir/../date/duminici_variabile_sc_bd.csv", "r" );
	$week = FALSE;
	while( ( $day = fgetcsv( $file, 80 ) ) !== FALSE )
	{
		assert( count( $day ) == 2 );
		if( $day[0] == $size )
		{
			$week = $day[1];
			break;
		}
	}
	fclose( $file );
	return $week;
}

function getBDWeek( $week, $Easter )
{
	$size = getBDSize( $Easter );
    $dir = dirname(__FILE__);
	$file = fopen("$dir/../date/duminici_variabile_bd.csv", "r" );
	$bd = FALSE;
	while( ( $day = fgetcsv( $file, 80 ) ) !== FALSE )
	{
		assert( count( $day ) == $day[0] + 1 );
		if( $day[0] == $size )
		{
			$bd = @$day[$week];
			break;
		}
	}
	fclose( $file );
	assert( $bd !== FALSE );
	return $bd;
}

function getBDSize( $Easter )
{
	$year = $Easter["year"];
	$bd = mktime( 0, 0, 0, 1, 6, $year );
	$bd = getdate( $bd );
	$day = 6 + ( 7 - $bd["wday"] ) + 7;
	$dayEaster = $Easter["yday"];
	$dayTriod = $dayEaster - 10 * 7 + 1;
	$size = ( $dayTriod - $day ) / 7;
	assert( $size == floor( $size ) );
	return $size;
}

function isLeap( $year )
{
	return $year % 4 == 0 && ($year % 100 != 0 || $year % 400 == 0);
}
?>